@@ -1,8 +1,8 @@
 # Paranoid::Args -- Command-line argument parsing functions
 #
-# (c) 2005, Arthur Corliss <corliss@digitalmages.com>
+# (c) 2005 - 2015, Arthur Corliss <corliss@digitalmages.com>
 #
-# $Id: Args.pm,v 0.22 2010/04/15 23:23:28 acorliss Exp $
+# $Id: lib/Paranoid/Args.pm, 2.01 2016/06/23 00:34:49 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -17,21 +17,20 @@
 
 package Paranoid::Args;
 
-use 5.006;
+use 5.008;
 
 use strict;
 use warnings;
 use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use base qw(Exporter);
-use Carp;
 use Paranoid;
 use Paranoid::Debug qw(:all);
 
-($VERSION) = ( q$Revision: 0.22 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 2.01 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT      = qw(parseArgs);
-@EXPORT_OK   = qw(parseArgs);
-%EXPORT_TAGS = ( all => [qw(parseArgs)], );
+@EXPORT_OK   = @EXPORT;
+%EXPORT_TAGS = ( all => [@EXPORT_OK], );
 
 #####################################################################
 #
@@ -57,7 +56,7 @@ use Paranoid::Debug qw(:all);
     # Internal errors array
     my @errors;
 
-    sub _resetErrors () {
+    sub _resetErrors {
 
         # Purpose:  Empties @errors
         # Returns:  True (1)
@@ -67,7 +66,7 @@ use Paranoid::Debug qw(:all);
         return 1;
     }
 
-    sub _pushErrors ($) {
+    sub _pushErrors {
 
         # Purpose:  Pushes a new string onto the @errors array
         # Returns:  Same argument as called with
@@ -78,7 +77,7 @@ use Paranoid::Debug qw(:all);
         return $message;
     }
 
-    sub listErrors () {
+    sub listErrors {
 
         # Purpose:  Gets the contents of @errors
         # Returns:  Contents of @errors
@@ -90,7 +89,7 @@ use Paranoid::Debug qw(:all);
     # Internal options hash
     my %options;
 
-    sub _getOption ($) {
+    sub _getOption {
 
         # Purpose:  Gets the template associated with passed option
         # Returns:  Reference to template hash or undef should the
@@ -102,7 +101,7 @@ use Paranoid::Debug qw(:all);
         return exists $options{$option} ? $options{$option} : undef;
     }
 
-    sub _setOption ($$) {
+    sub _setOption {
 
         # Purpose:  Associates the passed option to the passed template in
         #           %options
@@ -117,7 +116,7 @@ use Paranoid::Debug qw(:all);
         return 1;
     }
 
-    sub _optionsKeys () {
+    sub _optionsKeys {
 
         # Purpose:  Returns a list of keys from %options
         # Returns:  keys %options
@@ -126,7 +125,7 @@ use Paranoid::Debug qw(:all);
         return keys %options;
     }
 
-    sub _resetOptions () {
+    sub _resetOptions {
 
         # Purpose:  Empties the %options
         # Returns:  True (1)
@@ -140,7 +139,7 @@ use Paranoid::Debug qw(:all);
     # Internal arguments list
     my @arguments;
 
-    sub _getArgRef () {
+    sub _getArgRef {
 
         # Purpose:  Gets a reference the argument array
         # Returns:  Array reference
@@ -149,7 +148,7 @@ use Paranoid::Debug qw(:all);
         return \@arguments;
     }
 
-    sub clearMemory () {
+    sub clearMemory {
 
         # Purpose:  Empties all internal data structures
         # Returns:  True (1)
@@ -164,7 +163,7 @@ use Paranoid::Debug qw(:all);
     }
 }
 
-sub _tLint ($) {
+sub _tLint {
 
     # Purpose:  Performs basic checks on a given option template for
     #           correctness
@@ -175,7 +174,7 @@ sub _tLint ($) {
     my $rv   = 1;
     my ( $oname, @at );
 
-    pdebug( "entering w/($tref)", PDLEVEL2 );
+    pdebug( 'entering w/(%s)', PDLEVEL2, $tref );
     pIn();
 
     # Get the option name for reporting purposes (should have been populated
@@ -199,7 +198,7 @@ sub _tLint ($) {
     # Make sure the template contains only supported characters
     if ($rv) {
         unless ( defined $$tref{Template}
-            && $$tref{Template} =~ /^[\$\@]*$/sm ) {
+            && $$tref{Template} =~ /^[\$\@]*$/s ) {
             _pushErrors( "$oname option declared with an invalid template"
                     . "($$tref{Template})" );
             $rv = 0;
@@ -209,14 +208,14 @@ sub _tLint ($) {
     # Make sure option names are sane
     if ($rv) {
         if ( defined $$tref{Short} ) {
-            unless ( $$tref{Short} =~ /^[a-zA-Z0-9]$/sm ) {
+            unless ( $$tref{Short} =~ /^[a-zA-Z0-9]$/s ) {
                 _pushErrors(
                     "Invalid name for the short option ($$tref{Short})");
                 $rv = 0;
             }
         }
         if ( defined $$tref{Long} ) {
-            unless ( $$tref{Long} =~ /^[a-zA-Z0-9-]{2,}$/sm ) {
+            unless ( $$tref{Long} =~ /^[a-zA-Z0-9-]{2,}$/s ) {
                 _pushErrors(
                     "Invalid name for the long option ($$tref{Long})");
                 $rv = 0;
@@ -228,7 +227,7 @@ sub _tLint ($) {
     # set to allow bundling
     if ($rv) {
         if ( $$tref{Template} =~ /\@/sm ) {
-            @at = ( $$tref{Template} =~ m#(\@)#smg );
+            @at = ( $$tref{Template} =~ m#(\@)#sg );
             if ( @at > 1 ) {
                 _pushErrors( 'The \'@\' symbol can only be used once in the '
                         . "template for $oname: $_" );
@@ -283,18 +282,18 @@ sub _tLint ($) {
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL2 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL2, $rv );
 
     return $rv;
 }
 
-sub _getArgs ($$$) {
+sub _getArgs ($$\@) {
 
     # Purpose:  Takes passed argument template and extracts the requisite
     #           arguments to satisfy it from the argument list.  The
     #           results are stored in the passed option list.
     # Results:  True (1) if successful, False (0) if not
-    # Usage:    $rv = _getArgs($option, $argTemplate, \@optionArgs);
+    # Usage:    $rv = _getArgs($option, $argTemplate, @optionArgs);
 
     my $option      = shift;         # Option name
     my $argTemplate = shift;         # Option argument template
@@ -303,13 +302,14 @@ sub _getArgs ($$$) {
     my $argRef      = _getArgRef();
     my @tmp;
 
-    pdebug( "entering w/($option)($argTemplate)($lref)", PDLEVEL2 );
+    pdebug( 'entering w/(%s)(%s)(%s)',
+        PDLEVEL2, $option, $argTemplate, $lref );
     pIn();
 
     # Empty the array
     @$lref = ();
 
-    pdebug( "contents of args: @$argRef", PDLEVEL4 );
+    pdebug( 'contents of args: %s', PDLEVEL4, @$argRef );
 
     # Start checking the contents of $argTemplate
     if ( $argTemplate eq '' ) {
@@ -317,7 +317,7 @@ sub _getArgs ($$$) {
         # Template is '' (boolean option)
         @$lref = (1);
 
-    } elsif ( $argTemplate =~ /\@/sm ) {
+    } elsif ( $argTemplate =~ /\@/s ) {
 
         # Template has a '@' in it -- we'll need to
         # grab as many of the next arguments as possible.
@@ -332,7 +332,7 @@ sub _getArgs ($$$) {
         } else {
 
             # False: gobble up to the next option-looking thing
-            while ( @$argRef and $$argRef[0] !~ /^--?(?:\w+.*)?$/sm ) {
+            while ( @$argRef and $$argRef[0] !~ /^--?(?:\w+.*)?$/s ) {
                 push @$lref, shift @$argRef;
             }
 
@@ -364,7 +364,7 @@ sub _getArgs ($$$) {
 
             # False:  grab as many non-option-looking things as we can
             while ( @$argRef
-                and $$argRef[0] !~ /^--?(?:\w+.*)$/sm
+                and $$argRef[0] !~ /^--?(?:\w+.*)$/s
                 and @$lref < length $argTemplate ) {
                 push @$lref, shift @$argRef;
             }
@@ -386,14 +386,14 @@ sub _getArgs ($$$) {
 
     # Final check:  did we get minimum requisite number of arguments?
     if ( @$lref < length $argTemplate ) {
-        pdebug(
-            _pushErrors(
-                "Missing the minimum number of arguments for $option"),
-            PDLEVEL1
-            );
+        _pushErrors(
+            pdebug(
+                'Missing the minimum number of arguments for %s', PDLEVEL1,
+                $option
+                ) );
         $rv = 0;
     } else {
-        pdebug( "extracted the following arguments: @$lref", PDLEVEL3 );
+        pdebug( 'extracted the following arguments: %s', PDLEVEL3, @$lref );
     }
 
     # sublist '@' portions of multicharacter templates
@@ -401,12 +401,12 @@ sub _getArgs ($$$) {
         @tmp = ( [], [], [] );
 
         # First, shift off all preceding '$'s
-        if ( $argTemplate =~ /^(\$+)/sm ) {
+        if ( $argTemplate =~ /^(\$+)/s ) {
             @{ $tmp[0] } = splice @$lref, 0, length $1;
         }
 
         # Next, pop off all trailing '$'
-        if ( $argTemplate =~ /(\$+)\$/sm ) {
+        if ( $argTemplate =~ /(\$+)\$/s ) {
             @{ $tmp[2] } = splice @$lref, -1 * length $1;
         }
 
@@ -419,31 +419,31 @@ sub _getArgs ($$$) {
         push @$lref, $tmp[1];
         push @$lref, @{ $tmp[2] } if @{ $tmp[2] };
 
-        pdebug( "sublisted arguments into: @$lref", PDLEVEL3 );
+        pdebug( 'sublisted arguments into: %s', PDLEVEL3, @$lref );
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL2 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL2, $rv );
 
     return $rv;
 }
 
-sub _storeArgs ($$$) {
+sub _storeArgs ($$\@) {
 
     # Purpose:  Stores the passed option arguments in the passed option
     #           template's Value, but in accordance with parameters in the
     #           template
     # Returns:  True (1)
-    # Usage:    _storeArgs($optionTemplate, $argTemplate, \@optionArgs);
+    # Usage:    _storeArgs($optionTemplate, $argTemplate, @optionArgs);
 
     my $tref        = shift;
     my $argTemplate = shift;
     my $lref        = shift;
 
-    pdebug( "entering w/($tref)($argTemplate)($lref)", PDLEVEL2 );
+    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL2, $tref, $argTemplate, $lref );
     pIn();
 
-    pdebug( "adding values to $$tref{Name}", PDLEVEL3 );
+    pdebug( 'adding values to %s', PDLEVEL3, $$tref{Name} );
 
     # Increment our usage counter
     $$tref{Count}++;
@@ -454,7 +454,7 @@ sub _storeArgs ($$$) {
         # Template is ''
         $$tref{Value} = 0 unless defined $$tref{Value};
         $$tref{Value}++;
-        pdebug( "Value is now $$tref{Value}", PDLEVEL3 );
+        pdebug( 'Value is now %s', PDLEVEL3, $$tref{Value} );
 
     } elsif ( $argTemplate eq '$' ) {
 
@@ -464,7 +464,7 @@ sub _storeArgs ($$$) {
             # Store the value directly since we
             # can only be used once
             $$tref{Value} = $$lref[0];
-            pdebug( "Value is now $$tref{Value}", PDLEVEL3 );
+            pdebug( 'Value is now %s', PDLEVEL3, $$tref{Value} );
 
         } else {
 
@@ -474,8 +474,7 @@ sub _storeArgs ($$$) {
                 unless defined $$tref{Value}
                     and ref $$tref{Value} eq 'ARRAY';
             push @{ $$tref{Value} }, $$lref[0];
-            pdebug( 'Value is now ' . join( ', ', @{ $$tref{Value} } ),
-                PDLEVEL3 );
+            pdebug( 'Value is now %s', PDLEVEL3, @{ $$tref{Value} } );
         }
 
     } else {
@@ -486,8 +485,7 @@ sub _storeArgs ($$$) {
             # Store the values directly in a an array
             # since we can only be used once
             $$tref{Value} = [@$lref];
-            pdebug( 'Value is now ' . join( ', ', @{ $$tref{Value} } ),
-                PDLEVEL3 );
+            pdebug( 'Value is now %s', PDLEVEL3, @{ $$tref{Value} } );
 
         } else {
 
@@ -497,8 +495,8 @@ sub _storeArgs ($$$) {
                 unless defined $$tref{Value}
                     and ref $$tref{Value} eq 'ARRAY';
             push @{ $$tref{Value} }, [@$lref];
-            pdebug( "Value now has @{[ scalar @{ $$tref{Value} } ]} sets",
-                PDLEVEL3 );
+            pdebug( 'Value now has %d sets',
+                PDLEVEL3, scalar @{ $$tref{Value} } );
         }
     }
 
@@ -508,36 +506,25 @@ sub _storeArgs ($$$) {
     return 1;
 }
 
-sub parseArgs ($$;$) {
+sub parseArgs (\@\%;\@) {
 
     # Purpose:  Extracts and validates all command-line arguments and options,
     #           storing them in an organized hash for easy retrieval
     # Returns:  True (1) if successful, False (0) if not
-    # Usage:    $rv = parseArgs(\@templates, \%options);
-    # Usage:    $rv = parseArgs(\@templates, \%options, \@args);
+    # Usage:    $rv = parseArgs(@templates, %options);
+    # Usage:    $rv = parseArgs(@templates, %options, @args);
 
     my $tlref = shift;    # Templates list ref
     my $oref  = shift;    # Options hash ref
     my $paref = shift;    # Program argument list ref
     my $rv    = 1;
-    my ( $aopt, $tref, $oname, $argRef, $arg, $argTemplate );
+    my ( $tref, $oname, $argRef, $arg, $argTemplate );
     my ( @tmp, @oargs, $regex );
 
     # Validate arguments
-    croak 'Mandatory first argument must be a list reference'
-        unless defined $tlref and ref $tlref eq 'ARRAY';
-    croak 'Mandatory second argument must be a hash reference'
-        unless defined $oref and ref $oref eq 'HASH';
-    if ( defined $paref ) {
-        croak 'Optional third argument must be a list reference'
-            unless defined $paref and ref $paref eq 'ARRAY';
-        $aopt = $paref;
-    } else {
-        $paref = \@ARGV;
-        $aopt  = 'undef';
-    }
+    $paref = \@ARGV unless defined $paref;
 
-    pdebug( "entering w/($tlref)($oref)($aopt)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $tlref, $oref, $paref );
     pIn();
 
     # Clear all internal data structures and reset flag
@@ -593,12 +580,10 @@ sub parseArgs ($$;$) {
             if ( defined _getOption( $$tref{Short} ) ) {
 
                 # It is -- report the error
-                Paranoid::ERROR = pdebug(
-                    _pushErrors(
-                        "the $$tref{Short} option has more than one template"
-                        ),
-                    PDLEVEL1
-                    );
+                Paranoid::ERROR = _pushErrors(
+                    pdebug(
+                        'the %s option has more than one template',
+                        PDLEVEL1, $$tref{Short} ) );
                 $rv = 0;
 
             } else {
@@ -615,12 +600,10 @@ sub parseArgs ($$;$) {
             if ( defined _getOption( $$tref{Long} ) ) {
 
                 # It is -- report the error
-                Paranoid::ERROR = pdebug(
-                    _pushErrors(
-                        "the $$tref{Long} option has more than one template"
-                        ),
-                    PDLEVEL1
-                    );
+                Paranoid::ERROR = _pushErrors(
+                    pdebug(
+                        'the %s option has more than one template',
+                        PDLEVEL1, $$tref{Long} ) );
                 $rv = 0;
 
             } else {
@@ -646,18 +629,18 @@ sub parseArgs ($$;$) {
                 # $arg is '--', so set the no options flag
                 _NOOPTIONS = 1;
 
-            } elsif ( not _NOOPTIONS and $arg =~ /^--?/sm ) {
+            } elsif ( not _NOOPTIONS and $arg =~ /^--?/s ) {
 
                 # '--' hasn't been passed yet and this looks
                 # like an option...
 
                 # Test types of options
-                if ( $arg =~ /^-(\w.*)$/sm ) {
+                if ( $arg =~ /^-(\w.*)$/s ) {
 
                     # With a single '-' it should be a short option.  However,
                     # we'll split the option portion, in case there's more
                     # than one character
-                    @tmp = split //sm, $1;
+                    @tmp = split //s, $1;
 
                     # If there's more than one character for the option name
                     # it must be either a bunch of bundled options or an
@@ -675,7 +658,7 @@ sub parseArgs ($$;$) {
                         and $$tref{Template} eq '$'
                         and not $$tref{CanBundle} ) {
                         unshift @$argRef, join '', @tmp[ 1 .. $#tmp ];
-                        splice @tmp, 1, 1;
+                        splice @tmp, 1;
                     }
 
                     # Start processing all remaining short options in @tmp
@@ -707,8 +690,8 @@ sub parseArgs ($$;$) {
                                     and $$tref{CountShort};
 
                             # Get any accompanying arguments
-                            unless (
-                                _getArgs( "-$_", $argTemplate, \@oargs ) ) {
+                            unless ( _getArgs( "-$_", $argTemplate, @oargs ) )
+                            {
                                 $rv = 0;
                                 next;
                             }
@@ -723,7 +706,7 @@ sub parseArgs ($$;$) {
                             }
 
                             # Store the values
-                            _storeArgs( $tref, $argTemplate, \@oargs );
+                            _storeArgs( $tref, $argTemplate, @oargs );
 
                         } else {
 
@@ -780,7 +763,7 @@ sub parseArgs ($$;$) {
 
                         # Snarf extra arguments
                         unless (
-                            _getArgs( "--$tmp[0]", $argTemplate, \@oargs ) ) {
+                            _getArgs( "--$tmp[0]", $argTemplate, @oargs ) ) {
                             $rv = 0;
                             next;
                         }
@@ -795,7 +778,7 @@ sub parseArgs ($$;$) {
                         }
 
                         # Store the values
-                        _storeArgs( $tref, $argTemplate, \@oargs );
+                        _storeArgs( $tref, $argTemplate, @oargs );
 
                     } else {
 
@@ -856,12 +839,10 @@ sub parseArgs ($$;$) {
 
         # Copy the values into %$oref
         $$oref{$_} = $$tref{Value};
-
-        # TODO: pdebug output
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
@@ -872,18 +853,18 @@ __END__
 
 =head1 NAME
 
-Paranoid::Args - Command-linne argument parsing functions
+Paranoid::Args - Command-line argument parsing functions
 
 =head1 VERSION
 
-$Id: Args.pm,v 0.22 2010/04/15 23:23:28 acorliss Exp $
+$Id: lib/Paranoid/Args.pm, 2.01 2016/06/23 00:34:49 acorliss Exp $
 
 =head1 SYNOPSIS
 
   use Paranoid::Args;
 
-  $rv = parseArgs(\@templates, \%opts);
-  $rv = parseArgs(\@templates, \%opts, \@args);
+  $rv = parseArgs(@templates, %opts);
+  $rv = parseArgs(@templates, %opts, @args);
 
   @errors = Paranoid::Args::listErrors();
   Paranoid::Args::clearMemory();
@@ -933,12 +914,9 @@ ensure correct usage of this module.
 =head2 PRESERVATION OF CONTEXT
 
 Simply put, preservation of context means remembering the order and grouping
-of associated arguments.  A demonstrative example would perhaps serve better
-than one of my poor explanations.
-
-Take the hypothetical case of "tagging" files.  The traditional approach is to
-define an option that takes a single string argument and apply them to the
-remaining contents of @ARGV:
+of associated arguments.  Take the hypothetical case of "tagging" files.  
+The traditional approach is to define an option that takes a single string 
+argument and apply them to the remaining contents of @ARGV:
 
   ./foo.pl -t "tag1" file1 file2
 
@@ -972,11 +950,6 @@ It's probably just me, but I find that a little easier to track.
 
 =head2 SUPPORTED COMMAND-LINE SYNTAX
 
-In keeping with my established tradition of discarding everything I have no
-use for this module does not support the same range of expressiveness that the
-B<Getopt::*> modules do.  Nor do we support "flexible" modes of differing
-modes of expressiveness.  What we do support we support unconditionally.
-
 The following list of syntactical options are supported:
 
 =over
@@ -1005,12 +978,12 @@ if they look like options.
 
 =back
 
-We don't support the hash key/value pairs (i.e., -s foo=one bar=two) or 
-argument type validation (B<Getopt::*> can validate string, integer, and 
-floating point argument types).  And while we support a short & long option 
-we don't support innumerable aliases in addition.  In short, if it isn't
-explicitly documented it isn't supported, though it probably is in
-B<Getopt::*>.
+This module don't support the hash key/value pairs (i.e., -s foo=one 
+bar=two) or argument type validation (B<Getopt::*> can validate string, 
+integer, and floating point argument types).  And while it supports a short 
+& long option it doesn't support innumerable aliases in addition.  In 
+short, if it isn't explicitly documented it isn't supported, though it 
+probably is in B<Getopt::*>.
 
 There are a few restrictions meant to eliminate confusion:
 
@@ -1037,8 +1010,8 @@ long as '@' is not part of the argument template.
 
 =head2 parseArgs
 
-  $rv = parseArgs(\@templates, \%opts);
-  $rv = parseArgs(\@templates, \%opts, \@args);
+  $rv = parseArgs(@templates, %opts);
+  $rv = parseArgs(@templates, %opts, @args);
 
 Using the option templates passed as the first reference this function 
 populates the options hash with all of the parsed options found in the 
@@ -1064,8 +1037,7 @@ in a list associated with the key B<PAYLOAD>.
 If you need a list of everything that was found wrong during a B<parseArgs>
 run, from template errors to command-line argument validation failures, you
 can get all of the messages form B<listErrors>.  Please note that we show it
-fully qualified about because it is B<not> exported under any circumstances.
-If you need these extended diagnostics, you'll need to call it as shown.
+fully qualified because it is B<not> exported.
 
 Each time B<parseArgs> is invoked this array is reset.
 
@@ -1075,7 +1047,7 @@ Each time B<parseArgs> is invoked this array is reset.
 
 If the existance of a (most likely) lightly populated array bothers you, you
 may use this function to empty all internal data structures of their contents.
-Like B<listErrors> this function is not exported under any circumstances.
+Like B<listErrors> this function is not exported.
 
 =head1 OPTION TEMPLATES
 
@@ -1300,7 +1272,7 @@ L<Paranoid::Debug>
     );
 
   # Process @ARGV:  -vvvfh host1 file1 file2 file3
-  if (parseArgs(\@otemplates, \%opts, \@errors)) {
+  if (parseArgs(@otemplates, %opts, @errors)) {
     setVerbosity($opts{'verbose'});
 
     if ($opts{'force'}) {
@@ -1335,5 +1307,5 @@ Arthur Corliss (corliss@digitalmages.com)
 This software is licensed under the same terms as Perl, itself. 
 Please see http://dev.perl.org/licenses/ for more information.
 
-(c) 2005, Arthur Corliss (corliss@digitalmages.com)
+(c) 2005 - 2015, Arthur Corliss (corliss@digitalmages.com)
 
